@@ -2,19 +2,19 @@ test_that("APAStyler works with linear models", {
   m0 <- stats::lm(mpg ~  1, data = mtcars)
   m1 <- stats::lm(mpg ~  hp, data = mtcars)
 
-  expect_invisible(APAStyler(m0))
-  expect_output(APAStyler(m1, file = ""))
+  expect_invisible(APAStyler(m0, print = FALSE))
+  expect_output(APAStyler(m1, file = "", print = FALSE))
 
   expect_type(
-    APAStyler(list(m0, m1)),
+    APAStyler(list(m0, m1), print = FALSE),
     "list")
 
   m2 <- stats::lm(mpg ~  factor(cyl), data = mtcars)
-  out <- APAStyler(m2)
+  out <- APAStyler(m2, print = FALSE)
   expect_true(inherits(out, "matrix"))
   expect_equal(nrow(out), 5)
 
-  out <- APAStyler(m2, pcontrol = list(stars = FALSE))
+  out <- APAStyler(m2, pcontrol = list(stars = FALSE), print = FALSE)
   expect_true(inherits(out, "matrix"))
   expect_equal(nrow(out), 5)
 })
@@ -33,12 +33,12 @@ test_that("APAStyler works with vglm model tests", {
                   data = JWileymisc_Sample_Test_Data)
   mt <- modelTest(m)
 
-  out <- APAStyler(mt)
+  out <- APAStyler(mt, print = FALSE)
   expect_s3_class(out, "data.table")
 
   ## works without stars too
   out <- APAStyler(mt, pcontrol = list(
     digits = 3, includeP = TRUE, includeSign = TRUE,
-    dropLeadingZero = TRUE, stars = FALSE))
+    dropLeadingZero = TRUE, stars = FALSE), print = FALSE)
   expect_s3_class(out, "data.table")
 })
